import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Spin, notification, Col, Row, Modal } from 'antd'

import Api from '@/api'
import UtilsDM from '@/utils/utils-datamanage.js'
import asyncComponent from '@/utils/asyncComponent'
import MKEmitter from '@/utils/events.js'
import TimerTask from '@/utils/timer-task.js'
import './index.scss'

const CardItem = asyncComponent(() => import('../cardItem'))
const NormalHeader = asyncComponent(() => import('@/tabviews/custom/components/share/normalheader'))

class PropCard extends Component {
  static propTpyes = {
    config: PropTypes.object
  }

  state = {
    BID: '',                   // 上级ID
    config: null,              // 图表配置信息
    loading: false,            // 数据加载状态
    activeKey: '',             // 选中数据
    data: {},                  // 数据
    BData: '',
    selected: 'false',
  }

  loaded = false
  autoTimer = null

  UNSAFE_componentWillMount () {
    const { config } = this.props

    let _config = fromJS(config).toJS()
    let _data = { $$empty: true }
    let BID = ''
    let BData = ''

    if (_config.setting.supModule) {
      BData = window.GLOB.CacheData.get(_config.setting.supModule)
    } else {
      BData = window.GLOB.CacheData.get(_config.$pageId)
    }
    if (BData) {
      BID = BData.$BID || ''
    }
    
    if (_config.wrap.datatype === 'dynamic') {
      _config.setting.onload = _config.setting.sync === 'true' ? 'false' : 'true'

      if (_config.setting.sync === 'true' && window.GLOB.SyncData.has(_config.dataName)) {
        _data = window.GLOB.SyncData.get(_config.dataName) || []

        if (_config.$cache) {
          Api.writeCacheConfig(_config.uuid, fromJS(_data).toJS())
        }

        _config.setting.sync = 'false'

        _data = _data[0] || {$$empty: true}

        this.loaded = true

        window.GLOB.SyncData.delete(_config.dataName)
      }
    } else if (_config.wrap.datatype === 'public' && window.GLOB.CacheData.get(_config.wrap.publicId)) {
      _data = window.GLOB.CacheData.get(_config.wrap.publicId)
      _data = fromJS(_data).toJS()
      this.loaded = true
    } else if (_config.wrap.datatype === 'static') {
      this.loaded = true
    }

    _data.$$BID = BID || ''
    _data.$$BData = BData || ''
    if (_config.setting.primaryKey) {
      _data.$$uuid = _data[_config.setting.primaryKey] || ''
    }

    _config.subcards.forEach(card => {
      if (card.setting.click === 'button' && !card.setting.linkbtn) {
        card.elements.forEach(ele => {
          if (ele.eleType === 'button') {
            card.setting.linkbtn = ele.uuid
          }
        })
        if (!card.setting.linkbtn) {
          card.setting.click = ''
        }
      }

      if (card.style.clear === 'left') {
        card.wStyle = {clear: 'left'}
      } else if (card.style.clear === 'right') {
        card.wStyle = {float: 'right'}
      }

      delete card.style.clear
    })

    let selected = _config.wrap.selected || 'false'

    _config.wrap.selStyle = _config.wrap.selStyle || 'active'
    _config.wrap.priKeyType = _config.wrap.priKeyType || 'static'
    _config.wrap.scale = _config.wrap.scale === 'true' ? 'scale' : ''

    _config.wrap.wrapClass =  `${_config.wrap.selStyle} ${_config.wrap.cardType || ''} ${_config.wrap.scale || ''} ${_config.wrap.layout || 'grid'}-layout float-${_config.wrap.cardFloat || 'left'}`

    if (_config.wrap.display === 'hidden') {
      _config.style.display = 'none'
    }

    this.setState({
      selected,
      data: _data,
      BID: BID || '',
      BData: BData || '',
      config: _config,
      arr_field: _config.columns.map(col => col.field).join(',')
    })
  }

  componentDidMount () {
    const { config } = this.state

    MKEmitter.addListener('reloadData', this.reloadData)
    MKEmitter.addListener('refreshLineData', this.refreshLineData)
    MKEmitter.addListener('resetSelectLine', this.resetParentParam)
    MKEmitter.addListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.addListener('refreshByButtonResult', this.refreshByButtonResult)

    if (config.setting.useMSearch) {
      MKEmitter.addListener('searchRefresh', this.searchRefresh)
    }

    if (config.setting.sync === 'true') {
      MKEmitter.addListener('transferSyncData', this.transferSyncData)
    }

    if (config.wrap.datatype === 'public') {
      MKEmitter.addListener('mkPublicData', this.mkPublicData)
    }

    if (config.timer && config.wrap.datatype === 'dynamic') {
      this.timer = new TimerTask()
      this.timer.init(config.uuid, config.timer, config.timerRepeats, () => {this.loadData(true)})
    }

    this.initExec()
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('reloadData', this.reloadData)
    MKEmitter.removeListener('mkPublicData', this.mkPublicData)
    MKEmitter.removeListener('searchRefresh', this.searchRefresh)
    MKEmitter.removeListener('refreshLineData', this.refreshLineData)
    MKEmitter.removeListener('resetSelectLine', this.resetParentParam)
    MKEmitter.removeListener('transferSyncData', this.transferSyncData)
    MKEmitter.removeListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.removeListener('refreshByButtonResult', this.refreshByButtonResult)

    this.timer && this.timer.stop()
  }

  initExec = () => {
    const { config, selected, data } = this.state

    if (config.wrap.datatype === 'dynamic') {
      if (config.$cache) {
        if (config.$time) {
          if (!this.loaded) {
            Api.getLCacheConfig(config.uuid, config.$time).then(res => {
              if (!res && config.setting.onload === 'true') {
                setTimeout(() => {
                  this.loadData()
                }, config.setting.delay || 0)
              }
    
              if (!res || this.loaded) return
  
              let _data = res[0] || {$$empty: true}
              _data.$$uuid = _data[config.setting.primaryKey] || ''
      
              this.setState({data: _data})
            })
          }
        } else {
          if (!this.loaded) {
            Api.getLCacheConfig(config.uuid, 0).then(res => {
              if (!res || this.loaded) return
      
              let _data = res[0] || {$$empty: true}
              _data.$$uuid = _data[config.setting.primaryKey] || ''
      
              this.setState({data: _data})
            })
          }
  
          if (config.setting.onload === 'true') {
            setTimeout(() => {
              this.loadData()
            }, config.setting.delay || 0)
          } else if (this.loaded) {
            if (config.wrap.goback === 'true' && data.$$empty) {
              this.timer && this.timer.stop()
    
              MKEmitter.emit('closeTabView', config.$pageId)
            } else {
              if (selected !== 'false') {
                this.checkTopLine()
              } else {
                this.transferLine()
              }
              this.autoExec()
            }
          }
        }
      } else if (config.setting.onload === 'true') {
        setTimeout(() => {
          this.loadData()
        }, config.setting.delay || 0)
      } else if (this.loaded) {
        if (config.wrap.goback === 'true' && data.$$empty) {
          this.timer && this.timer.stop()

          MKEmitter.emit('closeTabView', config.$pageId)
        } else {
          if (selected !== 'false') {
            this.checkTopLine()
          } else {
            this.transferLine()
          }
          this.autoExec()
        }
      }
    } else if (config.wrap.datatype === 'public') {
      if (this.loaded) {
        if (config.wrap.goback === 'true' && data.$$empty) {
          this.timer && this.timer.stop()
  
          MKEmitter.emit('closeTabView', config.$pageId)
        } else {
          if (selected !== 'false') {
            this.checkTopLine()
          } else if (data.$$uuid) {
            this.transferLine()
          }
          this.autoExec()
        }
      }
    } else {
      if (selected !== 'false') {
        setTimeout(() => {
          this.checkTopLine()
        }, 200)
      }

      if (config.$cache && config.$time && config.wrap.autoExec) {
        Api.getLCacheConfig(config.uuid, config.$time).then(res => {
          if (!res) {
            this.autoExec()
          } else {
            Api.writeCacheConfig(config.uuid, [])
          }
        })
      } else {
        this.autoExec()
      }
    }
  }

  transferSyncData = (syncId) => {
    const { config } = this.state

    if (config.$syncId !== syncId) return

    const { BID, BData, selected } = this.state

    let _data = window.GLOB.SyncData.get(config.dataName) || []

    if (config.$cache) {
      Api.writeCacheConfig(config.uuid, fromJS(_data).toJS())
    }

    _data = _data[0] || {$$empty: true}

    _data.$$BID = BID || ''
    _data.$$BData = BData || ''
    _data.$$uuid = _data[config.setting.primaryKey] || ''

    this.loaded = true

    this.setState({data: _data}, () => {
      if (config.wrap.goback === 'true' && _data.$$empty) {
        this.timer && this.timer.stop()

        MKEmitter.emit('closeTabView', config.$pageId)
      } else {
        if (selected !== 'false') {
          this.checkTopLine()
        } else if (_data.$$uuid) {
          this.transferLine()
        }
        this.autoExec()
      }
    })

    window.GLOB.SyncData.delete(config.dataName)

    MKEmitter.removeListener('transferSyncData', this.transferSyncData)
  }

  searchRefresh = (searchId) => {
    const { config } = this.state

    if (config.$searchId !== searchId) return
    
    this.setState({}, () => {
      this.loadData()
    })
  }

  mkPublicData = (publicId, data) => {
    const { config, selected, BID, BData } = this.state

    if (config.wrap.datatype === 'public' && config.wrap.publicId === publicId) {
      let _data = fromJS(data).toJS()

      _data.$$BID = BID || ''
      _data.$$BData = BData || ''
      _data.$$uuid = _data[config.setting.primaryKey] || ''

      this.loaded = true

      this.setState({data: _data}, () => {
        if (config.wrap.goback === 'true' && _data.$$empty) {
          this.timer && this.timer.stop()

          MKEmitter.emit('closeTabView', config.$pageId)
        } else {
          if (selected !== 'false') {
            this.checkTopLine()
          } else {
            this.transferLine()
          }
          this.autoExec()
        }
      })
    }
  }

  autoExec = (times) => {
    const { config, data } = this.state

    if (!config.wrap.autoExec) return

    let btn = document.getElementById('button' + config.wrap.autoExec)

    clearTimeout(this.autoTimer)

    if (btn) {
      MKEmitter.emit('triggerBtnId', config.wrap.autoExec, data.$$empty ? [] : [data])
    } else if (!times || times < 20) {
      times = times ? times + 1 : 1
      this.autoTimer = setTimeout(() => {
        this.autoExec(times)
      }, 1000)
    }
  }

  checkTopLine = () => {
    const { config, data, selected } = this.state

    this.setState({
      activeKey: 0,
      selected: selected === 'init' ? 'false' : selected
    })

    let primaryId = config.subcards[0].setting.primaryId || ''
    if (config.wrap.priKeyType === 'dynamic') {
      primaryId = data.$$uuid || ''
    } else if (config.wrap.priKeyType === 'joint') {
      primaryId = (data.$$uuid || '') + ',' + primaryId
    }

    MKEmitter.emit('resetSelectLine', config.uuid, primaryId, data)
  }

  transferLine = () => {
    const { config, data } = this.state

    if (config.wrap.cardType) return

    MKEmitter.emit('resetSelectLine', config.uuid, data.$$uuid || '', data)
  }

  /**
   * @description 按钮执行完成后页面刷新
   * @param {*} menuId     // 菜单Id
   * @param {*} position   // 刷新位置
   * @param {*} btn        // 执行的按钮
   */
  refreshByButtonResult = (menuId, position, btn) => {
    const { config, BID } = this.state

    if (config.uuid !== menuId) return

    if ((position === 'mainline' || position === 'popclose') && config.setting.supModule && BID) { // 刷新源组件时，附带刷新上级行与当前组件
      MKEmitter.emit('reloadData', config.setting.supModule, BID)
    } else {
      this.loadData(false, btn)
    }
  }

  refreshLineData = (menuId, btn, uuid, count) => {
    const { config, data } = this.state

    if (config.uuid !== menuId) return
    
    let _data = fromJS(data).toJS()
    _data[btn.field] = count

    this.setState({
      data: _data
    })
  }

  resetParentParam = (MenuID, id, data) => {
    const { config } = this.state

    if (!config.setting.supModule || config.setting.supModule !== MenuID) return
    
    if (id !== this.state.BID || id !== '') {
      this.setState({ BID: id, BData: data }, () => {
        this.loadData()
      })
    }
  }

  /**
   * @description 导出Excel时，获取页面搜索排序等参数
   */
  queryModuleParam = (menuId, callback) => {
    const { config } = this.state

    if (config.uuid !== menuId) return

    let searches = []
    if (config.setting.useMSearch) { // 主表搜索条件
      searches = window.GLOB.SearchBox.get(config.$searchId) || []
    }

    callback({
      arr_field: '',
      orderBy: '',
      search: searches,
      menuName: config.name
    })
  }

  reloadData = (menuId) => {
    const { config } = this.state

    if (menuId !== config.uuid) return

    this.loadData()
  }

  async loadData (hastimer, btn) {
    const { config, arr_field, BID, BData, selected } = this.state

    if (config.wrap.datatype === 'public') {
      MKEmitter.emit('reloadData', config.wrap.publicId)
      return
    }
    
    if (config.wrap.datatype === 'static') {
      this.setState({
        data: {$$BID: BID || '', $$BData: BData, $$empty: true},
      })

      if (!btn) {
        this.autoExec()
      }
      return
    } else if (config.setting.supModule && !BID) { // BID 不存在时，不做查询
      this.setState({
        data: {$$BID: BID || '', $$BData: BData, $$empty: true},
      })
      this.loaded = true
      return
    }

    let searches = []
    if (config.setting.useMSearch) { // 主表搜索条件
      searches = window.GLOB.SearchBox.get(config.$searchId) || []
    }

    if (config.$s_req && searches.filter(item => item.required && item.value === '').length > 0) {
      return
    }

    if (!hastimer) {
      this.setState({
        loading: true
      })
    }

    let _orderBy = config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, arr_field, searches, _orderBy, 1, 1, BID)

    let result = await Api.genericInterface(param)
    if (result.status) {
      let _data = {}

      this.loaded = true
      if (config.$cache && config.setting.onload !== 'false') {
        Api.writeCacheConfig(config.uuid, result.data || [])
      }

      if (!result.data || !result.data[0]) {
        _data.$$empty = true
      } else {
        _data = result.data[0]
      }

      _data.$$BID = BID || ''
      _data.$$BData = BData
      _data.$$uuid = _data[config.setting.primaryKey] || ''

      this.setState({
        data: _data,
        loading: false
      }, () => {
        if (config.wrap.goback === 'true' && _data.$$empty) {
          this.timer && this.timer.stop()

          MKEmitter.emit('closeTabView', config.$pageId)
        } else {
          if (selected !== 'false') {
            this.checkTopLine()
          } else {
            this.transferLine()
          }
          if (!btn) {
            this.autoExec()
          }
        }
      })

      if (config.timer && config.clearField && result.data && result.data[0]) {
        let vals = (config.clearValue || '').split(',')
        if (vals.includes(result.data[0][config.clearField])) {
          this.timer && this.timer.stop()
        }
      }
      if (result.message) {
        if (result.ErrCode === 'Y') {
          Modal.success({
            title: result.message
          })
        } else if (result.ErrCode === 'S') {
          notification.success({
            top: 92,
            message: result.message,
            duration: 2
          })
        }
      }
    } else {
      this.setState({
        loading: false
      })
      this.timer && this.timer.stop()

      if (!result.message) return
      if (result.ErrCode === 'N') {
        Modal.error({
          title: result.message,
        })
      } else if (result.ErrCode !== '-2') {
        notification.error({
          top: 92,
          message: result.message,
          duration: 10
        })
      }
    }
  }

  changeCard = (index, item) => {
    const { config, data, activeKey } = this.state

    if (!config.wrap.cardType || activeKey === index) return

    this.setState({
      activeKey: index
    })

    let primaryId = item.setting.primaryId || ''
    if (config.wrap.priKeyType === 'dynamic') {
      primaryId = data.$$uuid || ''
    } else if (config.wrap.priKeyType === 'joint') {
      primaryId = (data.$$uuid || '') + ',' + primaryId
    }

    MKEmitter.emit('resetSelectLine', config.uuid, primaryId, data)
  }

  render() {
    const { config, loading, data, activeKey } = this.state

    if (config.wrap.empty === 'hidden' && (!data || data.$$empty)) return null

    return (
      <div className="custom-prop-card-box" id={'anchor' + config.uuid} style={config.style}>
        {loading ?
          <div className="loading-mask">
            <div className="ant-spin-blur"></div>
            <Spin />
          </div> : null
        }
        <NormalHeader config={config}/>
        <Row className={`card-row-list data-zoom ${config.wrap.wrapClass}`}>
          {config.subcards.map((item, index) => {
            let className = 'mk-card '
            if (activeKey === index) {
              className += 'active'
            }

            return (
              <Col className={className} key={index} style={item.wStyle} span={item.setting.width || 6} offset={item.offset || 0}>
                <CardItem card={item} cards={config} data={data} onClick={() => {this.changeCard(index, item)}}/>
              </Col>
            )
          })}
        </Row>
      </div>
    )
  }
}

export default PropCard